<?php
/*+**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 ************************************************************************************/

class SMSNotifier_ISMS_Provider implements SMSNotifier_ISMSProvider_Model {

	private $userName;
	private $password;
	private $parameters = array();

	const SERVICE_URI = 'https://www.isms.com.my/isms_send.php';
	private static $REQUIRED_PARAMETERS = array('type', 'sendid');

	/**
	 * Function to get provider name
	 * @return <String> provider name
	 */
	public function getName() {
		return 'ISMS';
	}

	/**
	 * Function to get required parameters other than (userName, password)
	 * @return <array> required parameters list
	 */
	public function getRequiredParams() {
		return self::$REQUIRED_PARAMETERS;
	}

	/**
	 * Function to get service URL to use for a given type
	 * @param <String> $type like SEND, PING, QUERY
	 */
	public function getServiceURL($type = false) {
		if($type) {
			switch(strtoupper($type)) {
				case self::SERVICE_AUTH: return  self::SERVICE_URI . '/http/auth';
				case self::SERVICE_SEND: return  self::SERVICE_URI . '/http/sendmsg';
				case self::SERVICE_QUERY: return self::SERVICE_URI . '/http/querymsg';
			}
		}
		return false;
	}

	/**
	 * Function to set authentication parameters
	 * @param <String> $userName
	 * @param <String> $password
	 */
	public function setAuthParameters($userName, $password) {
		$this->userName = $userName;
		$this->password = $password;
	}

	/**
	 * Function to set non-auth parameter.
	 * @param <String> $key
	 * @param <String> $value
	 */
	public function setParameter($key, $value) {
		$this->parameters[$key] = $value;
	}

	/**
	 * Function to get parameter value
	 * @param <String> $key
	 * @param <String> $defaultValue
	 * @return <String> value/$default value
	 */
	public function getParameter($key, $defaultValue = false) {
		if(isset($this->parameters[$key])) {
			return $this->parameters[$key];
		}
		return $defaultValue;
	}

	/**
	 * Function to prepare parameters
	 * @return <Array> parameters
	 */
	protected function prepareParameters() {
		$params = array('un' => $this->userName, 'pwd' => $this->password);
		foreach (self::$REQUIRED_PARAMETERS as $key) {
			$params[$key] = $this->getParameter($key);
		}
		return $params;
	}

	/**
	 * Function to handle SMS Send operation
	 * @param <String> $message
	 * @param <Mixed> $toNumbers One or Array of numbers
	 */
	public function send($message, $toNumbers) {
		
		//showAlert("$message");
		
		if(!is_array($toNumbers)) {
			$toNumbers = array($toNumbers);
		}

		$params = $this->prepareParameters();
		$params['msg'] = $message;
		$params['dstno'] = implode(',', $toNumbers);

		$serviceURL = $this->getServiceURL(self::SERVICE_SEND);
		$httpClient = new Vtiger_Net_Client($serviceURL);
		$response = $httpClient->doPost($params);
		$responseLines = split("\n", $response);

		$results = array();
		$responseLine = $responseLine[0];
		foreach($toNumbers as $toNumber) {
			$responseLine = trim($responseLine);
			if(empty($responseLine)) $responseLine = "2000 = SUCCESS";

			$result = array( 'error' => false, 'statusmessage' => '' );
			if(preg_match("/\-([\d]{4}) = (.*)/", trim($responseLine), $matches)) {
				$result['error'] = true;
				$result['to'] = $toNumber;
				$result['statusmessage'] = $matches[0]; // Complete error message
			} else if(preg_match("/2000 = SUCCESS/", $responseLine, $matches)) {
				$result['id'] = "";
				$result['to'] = $toNumber;
				$result['status'] = "Delivered";
				$result['statusmessage'] = "2000 = SUCCESS";
			} else if(preg_match("/ID: (.*)/", $responseLine, $matches)) {
				$result['id'] = "";
				$result['to'] = $toNumbers[0];
				$result['status'] = self::MSG_STATUS_DELIVERED;
			}
			$results[] = $result;
		}
		return $results;
	}

	/**
	 * Function to get query for status using messgae id
	 * @param <Number> $messageId
	 */
	public function query($messageId) {
		$params = $this->prepareParameters();
		$params['apimsgid'] = $messageId;

		$serviceURL = $this->getServiceURL(self::SERVICE_QUERY);
		$httpClient = new Vtiger_Net_Client($serviceURL);
		$response = $httpClient->doPost($params);
		$response = trim($response);

		$result = array( 'error' => false, 'needlookup' => 1, 'statusmessage' => '' );

		if(preg_match("/ERR: (.*)/", $response, $matches)) {
			$result['error'] = true;
			$result['needlookup'] = 0;
			$result['statusmessage'] = $matches[0];
		} else if(preg_match("/ID: ([^ ]+) Status: ([^ ]+)/", $response, $matches)) {
			$result['id'] = trim($matches[1]);
			$status = trim($matches[2]);

			// Capture the status code as message by default.
			$result['statusmessage'] = "CODE: $status";
			if($status == '' || $status == '2000 = SUCCESS') {
				$result['status'] = self::MSG_STATUS_PROCESSING;
			} else if($status == '' || $status == '2000 = SUCCESS') {
				$result['status'] = self::MSG_STATUS_DISPATCHED;
				$result['needlookup'] = 0;
			} else {
				$statusMessage = '';
				switch($status) {
					case '001':
					$statusMessage = 'Message unknown';
					$needlookup = 0;
					break;
					case '005':
					$statusMessage = 'Error with message';
					$needlookup = 0;
					break;
					case '006':
					$statusMessage = 'User cancelled message delivery';
					$needlookup = 0;
					break;
					case '007':
					$statusMessage = 'Error delivering message';
					$needlookup = 0;
					break;
					case '009':
					$statusMessage = 'Routing error';
					$needlookup = 0;
					break;
					case '010':
					$statusMessage = 'Message expired';
					$needlookup = 0;
					break;
					case '012':
					$statusMessage = 'Out of credit';
					$needlookup = 0;
					break;
				}
				if(!empty($statusMessage)) {
					$result['error'] = true;
					$result['needlookup'] = $needlookup;
					$result['statusmessage'] = $statusMessage;
				}
			}
		}
		return $result;
	}
}
?>
